// CWebSearchPlugin.cpp : implementation file
//
//	Copyright (c) 1997-2006, Firetongue Software
//

#include "stdafx.h"
#include "ocdweb.h"
#include "CWebSearchPlugin.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define _MAX_SUFFIX	48
#define ENGINE_REGISTRATION_KEY	_T("SOFTWARE\\Firetongue\\DADB\\SearchEngines")

static CString _escape_amps(LPCTSTR pszText)
{
	CString str;
	LPTSTR pszDest = str.GetBuffer(_tcslen(pszText) * 2 + 1);
	while (*pszText)
	{
		if (*pszText == _T('&'))
			*pszDest++ = _T('&');
		*pszDest++ = *pszText++;
	}
	*pszDest = 0;
	str.ReleaseBuffer();
	return str;
}

static LONG GetRegKey(HKEY key, LPCTSTR subkey, LPTSTR retdata)
{
    HKEY hkey;
    LONG retval = RegOpenKeyEx(key, subkey, 0, KEY_QUERY_VALUE, &hkey);

    if (retval == ERROR_SUCCESS) {
        long datasize = MAX_PATH;
		TCHAR data[MAX_PATH];
		RegQueryValue(hkey, NULL, data, &datasize);
		lstrcpy(retdata,data);
		RegCloseKey(hkey);
    }

    return retval;
}

static HINSTANCE GotoURL(LPCTSTR url, int showcmd)
{
	CWaitCursor wc;
    TCHAR key[MAX_PATH + MAX_PATH];	

    // First try ShellExecute()
    HINSTANCE result = ShellExecute(NULL, _T("open"), url, NULL,NULL, showcmd);

    // If it failed, get the .htm regkey and lookup the program
    if ((UINT)result <= HINSTANCE_ERROR) {		
		
        if (GetRegKey(HKEY_CLASSES_ROOT, _T(".htm"), key) == ERROR_SUCCESS) {
            lstrcat(key, _T("\\shell\\open\\command"));

            if (GetRegKey(HKEY_CLASSES_ROOT,key,key) == ERROR_SUCCESS) {
                TCHAR *pos;
                pos = _tcsstr(key, _T("\"%1\""));
                if (pos == NULL) {                     // No quotes found
                    pos = strstr(key, _T("%1"));       // Check for %1, without quotes
                    if (pos == NULL)                   // No parameter at all...
                        pos = key+lstrlen(key)-1;
                    else
                        *pos = '\0';                   // Remove the parameter
                }
                else
                    *pos = '\0';                       // Remove the parameter

                lstrcat(pos, _T(" "));
                lstrcat(pos, url);
                result = (HINSTANCE) WinExec(key,showcmd);
            }
        }
	}
	  
    return result;
}

/////////////////////////////////////////////////////////////////////////////
// CWebSearchDlg

class CWebSearchDlg : public CDialog
{
private:
	CWebSearchPlugin *m_pPlugin;

// Construction
public:
	CWebSearchDlg(CWebSearchPlugin *pPlugin, CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(CWebSearchDlg)
	enum { IDD = IDD_WEB_SEARCH };
	CComboBox	m_wndEngine;
	CString	m_sTerm;
	int		m_nEngine;
	//}}AFX_DATA

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CWebSearchDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CWebSearchDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CWebSearchDlg::CWebSearchDlg(CWebSearchPlugin *pPlugin, CWnd* pParent /*=NULL*/)
	: CDialog(CWebSearchDlg::IDD, pParent)
	, m_pPlugin(pPlugin)
{
	//{{AFX_DATA_INIT(CWebSearchDlg)
	m_sTerm = _T("");
	m_nEngine = -1;
	//}}AFX_DATA_INIT
}


void CWebSearchDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWebSearchDlg)
	DDX_Control(pDX, IDC_ENGINE, m_wndEngine);
	DDX_CBString(pDX, IDC_TERM, m_sTerm);
	DDX_CBIndex(pDX, IDC_ENGINE, m_nEngine);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CWebSearchDlg, CDialog)
	//{{AFX_MSG_MAP(CWebSearchDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CWebSearchDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	for (int i = 0; i < m_pPlugin->m_engines.GetSize(); i ++)
		m_wndEngine.AddString(m_pPlugin->m_engines[i].m_strName);

	m_nEngine = AfxGetApp()->GetProfileInt(_T("Settings"), _T("SearchEngine"), m_pPlugin->m_nGoogle);
	if (m_nEngine < 0 || m_nEngine >= m_pPlugin->m_engines.GetSize())
		m_nEngine = m_pPlugin->m_nGoogle;

	UpdateData(FALSE);
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CWebSearchPlugin

CWebSearchPlugin::~CWebSearchPlugin()
{
	if (m_pHost)
		m_pHost->Release();
}

BEGIN_MESSAGE_MAP(CWebSearchPlugin, CCmdTarget)
	//{{AFX_MSG_MAP(CWebSearchPlugin)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

IMPLEMENT_IUNKNOWN(CWebSearchPlugin, OUIPlugin)

BEGIN_INTERFACE_MAP(CWebSearchPlugin, CCmdTarget)
	INTERFACE_PART(CWebSearchPlugin, IID_IOUIPlugin, OUIPlugin)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWebSearchPlugin message handlers

void CWebSearchPlugin::InitSearchEngines()
{
	if (m_bEnginesLoaded)
		return;

	HKEY hRoot;
	if (::RegOpenKey(HKEY_LOCAL_MACHINE, ENGINE_REGISTRATION_KEY, &hRoot) == ERROR_SUCCESS)
	{
		CString strName;
		DWORD dwIndex = 0;
		for (;;)
		{
			TCHAR szValueName[128];
			DWORD dwType, dwDataSize = 0, dwSize = sizeof(szValueName);

			LONG lRetCode = ::RegEnumValue(hRoot, dwIndex, szValueName, &dwSize,
									NULL, &dwType, NULL, &dwDataSize);
			if (lRetCode != ERROR_SUCCESS)
			{
				_ASSERTE(lRetCode == ERROR_NO_MORE_ITEMS);
				break;
			}

			if (dwType == REG_SZ && dwDataSize > 0)
			{
				LPBYTE pbData = (LPBYTE) malloc((dwDataSize + 1) * 2);
				if (ERROR_SUCCESS == ::RegQueryValueEx(hRoot, szValueName, NULL,
										&dwType, pbData, &dwDataSize))
				{
					CEngineInfo ei;
					ei.m_strName = szValueName;
					ei.m_strURL = (LPCTSTR)pbData;
					if (!ei.m_strName.IsEmpty() &&
						!ei.m_strURL.IsEmpty() &&
						ei.m_strURL.Find(_T("%1")) >= 0)
					{
						m_engines.Add(ei);
						if (ei.m_strName == _T("Google"))
							m_nGoogle = m_engines.GetSize() - 1;
					}
				}
				else
					_ASSERTE(!"RegQueryValueEx failed");
				free(pbData);
			}
			else
				_ASSERTE(!"wrong type or size of registry data");

			dwIndex ++;
		}
		::RegCloseKey(hRoot);
	}
	m_bEnginesLoaded = TRUE;
}

HRESULT CWebSearchPlugin::XOUIPlugin::Initialize(IOUIPluginHost *pHost)
{
	METHOD_PROLOGUE(CWebSearchPlugin, OUIPlugin);
	if (!pHost)
		return E_INVALIDARG;

	pThis->m_pHost = pHost;
	pThis->m_pHost->AddRef();

#if 0
	//Debug code
	DWORD dwID = 0;
	IODatabase *pDB;
	VERIFY_OK(pHost->GetDatabase(&pDB));
	UINT c = 0;
	VERIFY_OK(pDB->ListRecords(NULL, &c));
	if (c > 0)
	{
		DWORD *pdw = new DWORD[c];
		UINT b = c;
		VERIFY_OK(pDB->ListRecords(pdw, &b));
		_ASSERTE(b == c);
		dwID = pdw[0];
		delete pdw;
	}

	if (dwID)
	{
		IORecord *pRec;
		VERIFY_OK(pDB->OpenRecord(dwID, &pRec));
		UINT t;
		VERIFY_OK(pRec->GetNumTracks(&t));
		_ASSERTE(t > 0);
		pRec->Release();
	}

	pDB->Release();

	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, 1001, NULL));
#endif

	BSTR bstrTitle;

	bstrTitle = CString((LPCTSTR)IDS_THIS_ALBUM_ON_AMAZON).AllocSysString();

	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_FOLDERS_POPUP|O_ALBUM_LIST_POPUP|O_ALBUM_PAGE_POPUP, ID_GOTO_ASIN, bstrTitle));
	::SysFreeString(bstrTitle);

	bstrTitle = CString((LPCTSTR)IDS_WEB_SEARCH_SUBMENU).AllocSysString();
	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS|O_NEW_SUBMENU, 1, bstrTitle));
	::SysFreeString(bstrTitle);

	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, ID_WEB_SEARCH_TITLE, NULL));
	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, ID_WEB_SEARCH_ARTIST, NULL));
	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, ID_WEB_SEARCH_ARTIST_TITLE, NULL));
	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, ID_WEB_SEARCH_TRACK_ARTIST, NULL));
	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, ID_WEB_SEARCH_TRACK_ARTIST_TITLE, NULL));

	bstrTitle = CString((LPCTSTR)IDS_WEB_SEARCH_CUSTOM_TITLE).AllocSysString();
	VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, ID_WEB_SEARCH_CUSTOM, bstrTitle));
	::SysFreeString(bstrTitle);

	pThis->InitSearchEngines();

	if (pThis->m_engines.GetSize() > 0)
	{
		VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, 0, NULL));

		for (int i = 0; i < pThis->m_engines.GetSize(); i ++)
			VERIFY_OK(pThis->m_pHost->AddMenuItem(O_ALL_CONTEXTS, ID_WEB_SEARCH_ENGINE_FIRST + i, NULL));
	}

	return S_OK;
}

void CWebSearchPlugin::GetCommonArtistTitle(IOSelection *pSelection, CString &strArtist, CString &strTitle,
											CString &strTrackArtist, CString &strTrackTitle, CString &strASIN)
{
	strArtist.Empty();
	strTitle.Empty();
	strTrackArtist.Empty();
	strTrackTitle.Empty();
	strASIN.Empty();

	BOOL bDiffArtists = FALSE;
	BOOL bDiffTitles = FALSE;

	if (pSelection)
	{
		UINT cSize = 0;
		if (SUCCEEDED(pSelection->GetRecordIDs(NULL, &cSize)) && cSize > 0)
		{
			DWORD *pdwIDs = new DWORD[cSize];
			if (SUCCEEDED(pSelection->GetRecordIDs(pdwIDs, &cSize)))
			{
				IODatabase *pDatabase;
				COleVariant va;
				if (SUCCEEDED(m_pHost->GetDatabase(&pDatabase)))
				{
					// Iterate all records in selection and query each for
					// artist, title, etc.
					//
					for (UINT i = 0; i < cSize; i ++)
					{
						IORecord *pRecord;
						if (SUCCEEDED(pDatabase->OpenRecord(pdwIDs[i], &pRecord)))
						{
							if (i == 0)
							{
								if (strASIN.IsEmpty() &&
									SUCCEEDED(pRecord->GetProp(O_ASIN, 0, &va)))
								{
									va.ChangeType(VT_BSTR);
									strASIN = va.bstrVal;
									va.Clear();
								}
							}

							if (!bDiffArtists)
							{
								CString strThisArtist;
								if (SUCCEEDED(pRecord->GetProp(O_VARIOUS_ARTISTS, 0, &va)))
								{
									if (va.vt == VT_I4 && va.lVal)
									{
										strArtist.Empty();
										bDiffArtists = TRUE;
									}
									va.Clear();
								}
								if (!bDiffArtists &&
									SUCCEEDED(pRecord->GetProp(O_ARTIST, 0, &va)))
								{
									va.ChangeType(VT_BSTR);
									strThisArtist = va.bstrVal;
									va.Clear();
								}
								if (!bDiffArtists)
								{
									if (strArtist.IsEmpty())
										strArtist = strThisArtist;
									else
									if (strArtist.CompareNoCase(strThisArtist))
									{
										strArtist.Empty();
										bDiffArtists = TRUE;
									}
								}
							}
							if (!bDiffTitles)
							{
								CString strThisTitle;
								if (SUCCEEDED(pRecord->GetProp(O_TITLE, 0, &va)))
								{
									va.ChangeType(VT_BSTR);
									strThisTitle = va.bstrVal;
									va.Clear();
								}
								if (!bDiffTitles)
								{
									if (strTitle.IsEmpty())
										strTitle = strThisTitle;
									else
									if (strTitle.CompareNoCase(strThisTitle))
									{
										strTitle.Empty();
										bDiffTitles = TRUE;
									}
								}
							}
							pRecord->Release();

							if (bDiffArtists && bDiffTitles)
								break;
						}
					}
					pDatabase->Release();
				}
			}
			delete pdwIDs;
		}

		if (SUCCEEDED(pSelection->GetTracks(NULL, NULL, &cSize)) && cSize == 1)
		{
			DWORD dwID;
			UINT nTrack;
			if (SUCCEEDED(pSelection->GetTracks(&dwID, &nTrack, &cSize)))
			{
				IODatabase *pDatabase;
				COleVariant va;
				if (SUCCEEDED(m_pHost->GetDatabase(&pDatabase)))
				{
					IORecord *pRecord;
					if (SUCCEEDED(pDatabase->OpenRecord(dwID, &pRecord)))
					{
						COleVariant va;
						if (SUCCEEDED(pRecord->GetProp(O_ARTIST, nTrack, &va)))
						{
							va.ChangeType(VT_BSTR);
							strTrackArtist = va.bstrVal;
							va.Clear();
						}
						if (SUCCEEDED(pRecord->GetProp(O_TRACK_TITLE, nTrack, &va)))
						{
							va.ChangeType(VT_BSTR);
							strTrackTitle = va.bstrVal;
							va.Clear();
						}
						pRecord->Release();
					}
					pDatabase->Release();
				}
			}
			else
				_ASSERTE(0);

			strASIN.Empty();	//no ASIN for tracks
		}
	}

	if (!strTrackTitle.IsEmpty() && strTrackArtist.IsEmpty())
		strTrackArtist = strArtist;

	_ASSERTE(!bDiffArtists || strArtist.IsEmpty());
	_ASSERTE(!bDiffTitles || strTitle.IsEmpty());
}

void CWebSearchPlugin::GetSelectionStrings(IOSelection *pSelection, UINT uID,
										   CString &strSuffix, CString &strMenuText,
										   CString &strASIN)
{
	strSuffix.Empty();
	strMenuText.Empty();

	CString strArtist, strTitle, strTrackArtist, strTrackTitle;
	GetCommonArtistTitle(pSelection, strArtist, strTitle,
						strTrackArtist, strTrackTitle,
						strASIN);

	switch (uID)
	{
	case ID_WEB_SEARCH_TITLE:
		strSuffix = strTitle;
		break;
	case ID_WEB_SEARCH_ARTIST:
		strSuffix = strArtist;
		break;
	case ID_WEB_SEARCH_ARTIST_TITLE:
		if (!strTitle.IsEmpty() && !strArtist.IsEmpty())
		{
			strSuffix = strArtist + _T(" ") + strTitle;
			strMenuText = strArtist + _T(" + ") + strTitle;
		}
		break;
	case ID_WEB_SEARCH_TRACK_ARTIST:
		if (!strTrackArtist.IsEmpty() && strTrackArtist != strArtist)
			strSuffix = strTrackArtist;
		break;
	case ID_WEB_SEARCH_TRACK_ARTIST_TITLE:
		if (!strTrackArtist.IsEmpty() && !strTrackTitle.IsEmpty())
		{
			strSuffix = strTrackArtist + _T(" ") + strTrackTitle;
			strMenuText = strTrackArtist + _T(" + ") + strTrackTitle;
		}
		break;
	case ID_GOTO_ASIN:
		break;
	default:
		_ASSERTE(0);	//don't know what command is this
	}

	strSuffix.TrimLeft();
	strSuffix.TrimRight();

	if (strMenuText.IsEmpty())
		strMenuText = strSuffix;

	// Limit string length...

	if (strMenuText.GetLength() > _MAX_SUFFIX)
	{
		strMenuText = strMenuText.Left(_MAX_SUFFIX);
		int iLastSpace = strMenuText.ReverseFind(_T(' '));
		if (iLastSpace > 0)
			strMenuText = strMenuText.Left(iLastSpace);
		strMenuText += _T("...");
	}
}

HRESULT CWebSearchPlugin::XOUIPlugin::GetMenuState(UINT command, O_UICONTEXT context,
									IOSelection *pSelection, DWORD *pMenuState,
									BSTR *pbstrMenuText)
{
	METHOD_PROLOGUE(CWebSearchPlugin, OUIPlugin);

#if 0
	// Debug code
	if (command == 1001)
	{
		*pMenuState = 0;
		if (*pbstrMenuText)
			::SysFreeString(*pbstrMenuText);
		LPCTSTR psz = "Unknown context";
		switch (context)
		{
#define AA(x) case x: psz = #x; break;
		AA(O_MAIN_MENU)
		AA(O_FOLDERS_POPUP)
		AA(O_ALBUM_LIST_POPUP)
		AA(O_TRACK_LIST_POPUP)
		AA(O_ALBUM_PAGE_POPUP)
		AA(O_TRACK_PAGE_POPUP)
#undef AA
		}

		O_SELTYPE type = O_EMPTY;
		if (pSelection)
			pSelection->GetType(&type);

		LPCTSTR psz2 = "Unknown selection";
		switch (type)
		{
#define AA(x) case x: psz2 = #x; break;
		AA(O_EMPTY)
		AA(O_SINGLE_TRACK)
		AA(O_MULTIPLE_TRACKS)
		AA(O_SINGLE_ALBUM)
		AA(O_SINGLE_VOLUME)
		AA(O_MULTIPLE_VOLUMES)
#undef AA
		}

		*pbstrMenuText = (CString(psz) + CString("/") + CString(psz2)).AllocSysString();
		return S_OK;
	}
#endif

	if (command >= ID_WEB_SEARCH_ENGINE_FIRST &&
		command < ID_WEB_SEARCH_ENGINE_FIRST + (UINT)pThis->m_engines.GetSize())
	{
		UINT idx = command - ID_WEB_SEARCH_ENGINE_FIRST;
		if (*pbstrMenuText)
			::SysFreeString(*pbstrMenuText);
		*pbstrMenuText = CString(pThis->m_engines[idx].m_strName).AllocSysString();
		*pMenuState = 0;
		if (idx == AfxGetApp()->GetProfileInt(_T("Settings"), _T("SearchEngine"), pThis->m_nGoogle))
			*pMenuState = O_RADIO;		// selected search engine
		return S_OK;
	}

	if (command == ID_WEB_SEARCH_CUSTOM)
	{
		// always enabled
		*pMenuState = 0;
		return S_OK;
	}

	CString strSuffix, strMenuText, strAsin;
	pThis->GetSelectionStrings(pSelection, command, strSuffix, strMenuText, strAsin);

	if (command == ID_GOTO_ASIN)
	{
		if (strAsin.IsEmpty())
			*pMenuState = O_HIDDEN;
		else
			*pMenuState = 0;
		return S_OK;
	}

	if (strSuffix.IsEmpty())
	{
		_ASSERTE(strMenuText.IsEmpty());

		*pMenuState = O_HIDDEN;
	}
	else
	{
		_ASSERTE(!strMenuText.IsEmpty());

		CString strText;
		strText.Format(IDS_WEB_SEARCH_MENU_FMT, (LPCTSTR) _escape_amps(strMenuText));

		if (*pbstrMenuText)
			::SysFreeString(*pbstrMenuText);
		*pbstrMenuText = strText.AllocSysString();
	}

	return S_OK;
}

HRESULT CWebSearchPlugin::XOUIPlugin::Execute(UINT command, O_UICONTEXT context,
									IOSelection *pSelection, HWND hParent)
{
	METHOD_PROLOGUE(CWebSearchPlugin, OUIPlugin);
	_ASSERTE(::IsWindow(hParent));

	if (command >= ID_WEB_SEARCH_ENGINE_FIRST &&
		command < ID_WEB_SEARCH_ENGINE_FIRST + (UINT)pThis->m_engines.GetSize())
	{
		UINT idx = command - ID_WEB_SEARCH_ENGINE_FIRST;
		AfxGetApp()->WriteProfileInt(_T("Settings"), _T("SearchEngine"), idx);
		return S_OK;
	}

	CString strPhrase;
	if (command == ID_WEB_SEARCH_CUSTOM)
	{
		CWebSearchDlg dlg(pThis, hParent ? CWnd::FromHandle(hParent) : NULL);
		if (dlg.DoModal() == IDOK)
			strPhrase = dlg.m_sTerm;
	}
	else
	{
		CString strMenuText, strASIN;
		pThis->GetSelectionStrings(pSelection, command, strPhrase, strMenuText, strASIN);
		if (command == ID_GOTO_ASIN)
		{
			if (!strASIN.IsEmpty())
			{
				CString strURL;
				strURL.Format(_T("http://www.amazon.com/exec/obidos/ASIN/%s/orangesuitemu-20"), strASIN);
				::GotoURL(strURL, SW_SHOW);
			}
			return S_OK;
		}
	}

	if (!strPhrase.IsEmpty())
	{
		LPCTSTR pszPhrase = strPhrase;
		TCHAR *psz = (TCHAR*) _alloca((_tcslen(pszPhrase) + 1) * sizeof(TCHAR) * 3);
		TCHAR *psz1 = psz;
		TRACE1("'%s' -> ", pszPhrase);
		while (*pszPhrase)
		{
			if (*pszPhrase >= _T('a') && *pszPhrase <= _T('z') ||
				*pszPhrase >= _T('A') && *pszPhrase <= _T('Z') ||
				*pszPhrase >= _T('0') && *pszPhrase <= _T('9'))
				*psz1++ = *pszPhrase;
			else
			if (*pszPhrase == _T(' '))
				*psz1++ = _T('+');
			else
			if (*pszPhrase != _T('"'))
			{
				_stprintf(psz1, _T("%%%02X"), (unsigned char)*pszPhrase);
				_ASSERTE(_tcslen(psz1) == 3);
				psz1 += _tcslen(psz1);
			}
			pszPhrase ++;
		}
		*psz1 = 0;
		TRACE1("'%s'\n", psz);

		if (pThis->m_engines.GetSize() == 0)
			return E_UNEXPECTED;

		int nSearchEngine = AfxGetApp()->GetProfileInt(_T("Settings"), _T("SearchEngine"), pThis->m_nGoogle);
		if (nSearchEngine < 0 || nSearchEngine >= pThis->m_engines.GetSize())
			nSearchEngine = pThis->m_nGoogle;

		CString strURL = pThis->m_engines[nSearchEngine].m_strURL;
		strURL.Replace(_T("%1"), psz);

		::GotoURL(strURL, SW_SHOW);
		return S_OK;
	}

	return S_FALSE;
}
